@@ -1,23 +1,28 @@
+from __future__ import absolute_import
 from flask.json import _json
 from flask_sqlalchemy import DefaultMeta
 from flask import Response, request, render_template, g
 from functools import wraps
 from toolspy import deep_group, merge, add_kv_to_dict, boolify, all_subclasses
 import inspect
-from .json_encoder import json_encoder
-from .query_booster import QueryBooster
+
 from sqlalchemy.sql import sqltypes
 from decimal import Decimal
 import dateutil.parser
 import math
 from flask_sqlalchemy import Pagination
 import traceback
-from schemalite.core import validate_object
+from schemalite.core import validate_dict
 from schemalite.validators import is_a_type_of, is_a_list_of_types_of
 from sqlalchemy.orm import class_mapper
 from sqlalchemy.orm.query import Query
 from sqlalchemy import or_, and_
+
+from .json_encoder import json_encoder
+from .query_booster import QueryBooster
 from .utils import type_coerce_value
+import six
+from six.moves import zip
 
 
 RESTRICTED = ['limit', 'sort', 'orderby', 'groupby', 'attrs',
@@ -33,6 +38,17 @@ OPERATOR_FUNC = {
 }
 
 
+def get_request_json():
+    if 'json' in g:
+        return g.json
+    return request.get_json()
+
+def get_request_args():
+    if 'args' in g:
+        return g.args
+    return request.args
+
+
 def json_dump(obj):
     return _json.dumps(
         obj,
@@ -138,16 +154,14 @@ def serializable_list(
                 })
         return result
     else:
-        result_list = map(
-            lambda o: serialized_obj(
+        result_list = [serialized_obj(
                 o, attrs_to_serialize=attrs_to_serialize,
                 rels_to_expand=rels_to_expand,
                 group_listrels_by=group_listrels_by,
                 rels_to_serialize=rels_to_serialize,
                 key_modifications=key_modifications,
                 dict_struct=dict_struct,
-                dict_post_processors=dict_post_processors),
-            olist)
+                dict_post_processors=dict_post_processors) for o in olist]
         if keyvals_to_merge:
             result_list = [merge(obj_dict, kvdict)
                            for obj_dict, kvdict in
@@ -159,9 +173,7 @@ def serialized_list(olist, **kwargs):
     """
     Misnamed. Should be deprecated eventually.
     """
-    return map(
-        lambda o: serialized_obj(o, **kwargs),
-        olist)
+    return [serialized_obj(o, **kwargs) for o in olist]
 
 
 def structured(struct, wrap=True, meta=None, struct_key='result', pre_render_callback=None):
@@ -330,7 +342,7 @@ ds_schema = {
     "fields": {
         "attrs": {
             "required": False,
-            "validators": [is_a_list_of_types_of(str, unicode)]
+            "validators": [is_a_list_of_types_of(str, six.text_type)]
         },
         "rels": {
             "required": False,
@@ -344,7 +356,7 @@ def _serializable_params(args, check_groupby=False):
     params = {}
     if '_ds' in args:
         params['dict_struct'] = _json.loads(args['_ds'])
-        if isinstance(params['dict_struct'], str) or isinstance(params['dict_struct'], unicode):
+        if isinstance(params['dict_struct'], str) or isinstance(params['dict_struct'], six.text_type):
             params['dict_struct'] = _json.loads(params['dict_struct'])
     if 'attrs' in args:
         attrs = args.get('attrs')
@@ -446,9 +458,11 @@ def as_list(func):
 #         value = dateutil.parser.parse(value).date()
 #     return value
 
-
-def modify_query_and_get_filter_function(query, keyword, value, op):
-    # print "in modify_query_and_get_filter_function ", query, keyword, value, op
+def return_joined_query_model_class_and_attr_name(query, keyword):
+    # print("in return_joined_query_model_class_and_attr_name")
+    # print(query, keyword)
+    # print("query.model_class ", query.model_class)
+    # print("query.criterion ", query._criterion)
     if '.' in keyword:
         kw_split_arr = keyword.split('.')
         prefix_names = kw_split_arr[:-1]
@@ -473,7 +487,12 @@ def modify_query_and_get_filter_function(query, keyword, value, op):
                         if r.key == rel_or_proxy_name)
                     model_class = mapped_rel.mapper.class_
                     if model_class not in [entity.class_ for entity in _query._join_entities]:
-                        _query = _query.join(model_class)
+                        # print("About to join model_class ", model_class)
+                        # print("rel_or_proxy_name is ", rel_or_proxy_name)
+                        # try:
+                        #     _query = _query.join(model_class)
+                        # except:
+                        _query = _query.join(rel_or_proxy_name)
                 elif rel_or_proxy_name in model_class.association_proxy_keys():
                     assoc_proxy = getattr(model_class, rel_or_proxy_name)
                     assoc_rel = next(
@@ -488,12 +507,14 @@ def modify_query_and_get_filter_function(query, keyword, value, op):
                     if model_class not in [entity.class_ for entity in _query._join_entities]:
                         _query = _query.join(model_class)
     else:
+        # print(". not in attr_name")
         model_class = query.model_class
         attr_name = keyword
         _query = query
         counter = 0  # to prevent infinite loop by some mistake
 
         while attr_name in model_class.association_proxy_keys() and counter < 10:
+            # print("attr_name in assoc proxy keys ", attr_name)
             counter += 1
             assoc_proxy = getattr(model_class, attr_name)
             assoc_rel = next(
@@ -502,6 +523,9 @@ def modify_query_and_get_filter_function(query, keyword, value, op):
             prev_model_class = model_class
             model_class = assoc_rel.mapper.class_
             attr_name = assoc_proxy.value_attr
+            # print("prev_model_class ", prev_model_class)
+            # print("model_class ", model_class)
+            # print("attr_name ", attr_name)
             if model_class not in [entity.class_ for entity in _query._join_entities]:
                 # _query = _query.join(model_class)
                 # Commenting this out because it is not possibel to directly
@@ -509,6 +533,15 @@ def modify_query_and_get_filter_function(query, keyword, value, op):
                 # Instead it is preferable to join by mentioning the relationshp
                 # itself.
                 _query = _query.join(getattr(prev_model_class, assoc_rel.key))
+    return (_query, model_class, attr_name)
+
+
+def modify_query_and_get_filter_function(query, keyword, value, op):
+    # print("in modify_query_and_get_filter_function")
+    _query, model_class, attr_name = return_joined_query_model_class_and_attr_name(query, keyword)
+    # print("in modify query, model_class ", model_class)
+    # print("in modify query, attr_name ", attr_name)
+    # print("in modify query, count ", _query.count())
 
     columns = getattr(
         getattr(model_class, '__mapper__'),
@@ -527,7 +560,9 @@ def modify_query_and_get_filter_function(query, keyword, value, op):
                 # else:
                 value = type_coerce_value(column_type, value)
     elif op == 'in':
-        value = map(lambda v: type_coerce_value(column_type, v), value)
+        value = [type_coerce_value(column_type, v) for v in value]
+
+    # print("in modify_query, value ", value)
 
     if hasattr(model_class, attr_name):
         return (_query, getattr(
@@ -561,17 +596,22 @@ def filter_query_with_key(query, keyword, value, op):
 
 
 def convert_filters_to_sqlalchemy_filter(query, filters, connector):
+    # print("in convert_filters_to_sqlalchemy_filter")
     sqfilters = []
     for f in filters:
+        # print("checking filter ", f)
         if "c" in f:
+            # print("found a connector c in filter")
             sub_sq_filter = convert_filters_to_sqlalchemy_filter(
                 query, f['f'], f['c'])
             if sub_sq_filter is not None:
                 sqfilters.append(sub_sq_filter)
         else:
+            # print("calling modify_query")
             query, sqfilter = modify_query_and_get_filter_function(
                 query, f["k"], f["v"], f["op"])
             sqfilters.append(sqfilter)
+    # print("found sqfilters list as ", sqfilters)
     if len(sqfilters) > 0:
         if connector == 'AND':
             return and_(*sqfilters)
@@ -624,6 +664,7 @@ def filter_query_using_filters_list(result, filters_dict):
     sqfilter = convert_filters_to_sqlalchemy_filter(result, filters, connector)
     if sqfilter is not None:
         result = result.filter(sqfilter)
+    # print("sqlalchemy filters ", result)
     return result
     # for f in filters:
     #     if "c" in f:
@@ -637,14 +678,10 @@ def filter_query_using_filters_list(result, filters_dict):
 
 def filter_query_using_args(result, args_to_skip=[]):
     if not (isinstance(result, Query) or isinstance(result, QueryBooster)):
-        # print "not an instance of Query"
-        # print "instance of ", type(result)
         if isinstance(result, DefaultMeta) and class_mapper(
                 result).polymorphic_on is not None:
-            # print "marking as polymorphic"
             result = result.query.with_polymorphic('*')
         else:
-            # print "marking as not polymorphic"
             result = result.query
     for kw in request.args:
         if kw not in args_to_skip:
@@ -665,7 +702,6 @@ def filter_query_using_args(result, args_to_skip=[]):
                     if value.lower() == 'none' or value.lower() == 'null' or value.strip() == '':
                         value = None
                     result = filter_query_with_key(result, kw, value, '=')
-    # print result
     return result
 
 
@@ -675,14 +711,18 @@ def fetch_results_in_requested_format(
     limit = request.args.get('limit', default_limit)
     sort = request.args.get('sort', default_sort)
     orderby = request.args.get('orderby') or default_orderby or 'id'
+
     offset = request.args.get('offset', None) or default_offset
     page = request.args.get('page', None) or default_page
     per_page = request.args.get('per_page') or default_per_page or PER_PAGE_ITEMS_COUNT
+
     if sort:
+        result, model_class, attr_name = return_joined_query_model_class_and_attr_name(result, orderby)
+        attr = getattr(model_class, attr_name)
         if sort == 'asc':
-            result = result.asc(orderby)
+            result = result.order_by(attr.asc())
         elif sort == 'desc':
-            result = result.desc(orderby)
+            result = result.order_by(attr.desc())
     if page:
         try:
             pagination = result.paginate(int(page), int(per_page))
@@ -727,12 +767,12 @@ def convert_result_to_response_structure(
             'curr_page_first_item_index': (page - 1) * per_page + 1,
             'curr_page_last_item_index': min(page * per_page, result.total)
         }
-        if isinstance(meta, dict) and len(meta.keys()) > 0:
+        if isinstance(meta, dict) and len(list(meta.keys())) > 0:
             pages_meta = merge(pages_meta, meta)
         return structured(
             serializable_list(result.items, **params_to_be_serialized),
             meta=pages_meta)
-    if isinstance(meta, dict) and len(meta.keys()) > 0:
+    if isinstance(meta, dict) and len(list(meta.keys())) > 0:
         kwargs = merge(params_to_be_serialized, {'meta': meta})
     else:
         kwargs = params_to_be_serialized
@@ -788,9 +828,12 @@ def process_args_and_fetch_rows(
 
     if '_f' in request.args:
         filters = _json.loads(request.args['_f'])
-        if isinstance(filters, str) or isinstance(filters, unicode):
+        if isinstance(filters, str) or isinstance(filters, six.text_type):
             filters = _json.loads(filters)
+        # print("filters are ", filters)
         q = filter_query_using_filters_list(q, filters)
+        # print("after applying filters")
+        # print(str(q))
 
     filtered_query = filter_query_using_args(q)
 
@@ -817,7 +860,7 @@ def process_args_and_render_json_list(q, **kwargs):
 
     if '_f' in request.args:
         filters = _json.loads(request.args['_f'])
-        if isinstance(filters, str) or isinstance(filters, unicode):
+        if isinstance(filters, str) or isinstance(filters, six.text_type):
             filters = _json.loads(filters)
         q = filter_query_using_filters_list(q, filters)
 
